@@ -41,6 +41,10 @@
 #include <QDBusMetaType>
 #endif
 
+#ifdef Q_OS_WIN
+#include "registry_win_p.h"
+#endif
+
 bool KConfigPrivate::mappingsRegistered = false;
 
 // For caching purposes
@@ -51,12 +55,12 @@ static QBasicMutex s_globalFilesMutex;
 Q_GLOBAL_STATIC_WITH_ARGS(QString, sGlobalFileName, (QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1String("/kdeglobals")))
 
 using ParseCacheKey = std::pair<QStringList, QString>;
+using ParseCacheTimestamp = QList<qint64>;
 struct ParseCacheValue {
     KEntryMap entries;
-    QDateTime parseTime;
+    ParseCacheTimestamp timestamp;
 };
-using ParseCache = QThreadStorage<QCache<ParseCacheKey, ParseCacheValue>>;
-Q_GLOBAL_STATIC(ParseCache, sGlobalParse)
+QThreadStorage<QCache<ParseCacheKey, ParseCacheValue>> sGlobalParse;
 
 #ifndef Q_OS_WIN
 static const Qt::CaseSensitivity sPathCaseSensitivity = Qt::CaseSensitive;
@@ -64,10 +68,29 @@ static const Qt::CaseSensitivity sPathCaseSensitivity = Qt::CaseSensitive;
 static const Qt::CaseSensitivity sPathCaseSensitivity = Qt::CaseInsensitive;
 #endif
 
+static QString getDefaultLocaleName()
+{
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
+    if (QLocale() == QLocale::system()) {
+        // If the default locale hasn't been changed then
+        // On Windows and Apple OSs, we cannot use QLocale::system() if an application-specific
+        // language was set by kxmlgui because Qt ignores LANGUAGE on Windows and Apple OSs.
+        if (const auto firstLanguage = qEnvironmentVariable("LANGUAGE").section(u':', 0, 0, QString::SectionSkipEmpty); !firstLanguage.isEmpty()) {
+            return firstLanguage;
+        }
+        // Also prefer the configured display language over the system language
+        if (const auto languages = QLocale::system().uiLanguages(); !languages.isEmpty()) {
+            // uiLanguages() uses dashes as separator, but KConfig assumes underscores
+            return languages.value(0).replace(u'-', u'_');
+        }
+    }
+#endif
+    return QLocale().name();
+}
+
 KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags, QStandardPaths::StandardLocation resourceType)
     : openFlags(flags)
     , resourceType(resourceType)
-    , mBackend(new KConfigIniBackend)
     , bDirty(false)
     , bReadDefaults(false)
     , bFileImmutable(false)
@@ -115,12 +138,12 @@ KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags, QStandardPaths::Standar
     //        mappingsRegistered = true;
     //    }
 
-    setLocale(QLocale().name());
+    setLocale(getDefaultLocaleName());
 }
 
 bool KConfigPrivate::lockLocal()
 {
-    return mBackend->lock();
+    return mBackend.lock();
 }
 
 static bool isGroupOrSubGroupMatch(KEntryMapConstIterator entryMapIt, const QString &group)
@@ -265,7 +288,7 @@ KConfig::KConfig(KConfigPrivate &d)
 KConfig::~KConfig()
 {
     Q_D(KConfig);
-    if (d->bDirty && d->mBackend->ref.loadRelaxed() == 1) {
+    if (d->bDirty) {
         sync();
     }
     delete d;
@@ -420,7 +443,7 @@ bool KConfig::sync()
         const QByteArray utf8Locale(locale().toUtf8());
 
         // Create the containing dir, maybe it wasn't there
-        d->mBackend->createEnclosing();
+        d->mBackend.createEnclosing();
 
         // lock the local file
         if (d->configState == ReadWrite && !d->lockLocal()) {
@@ -451,34 +474,34 @@ bool KConfig::sync()
         d->bDirty = false; // will revert to true if a config write fails
 
         if (d->wantGlobals() && writeGlobals) {
-            QExplicitlySharedDataPointer<KConfigIniBackend> tmp(new KConfigIniBackend());
-            tmp->setFilePath(*sGlobalFileName);
-            if (d->configState == ReadWrite && !tmp->lock()) {
+            KConfigIniBackend tmp;
+            tmp.setFilePath(*sGlobalFileName);
+            if (d->configState == ReadWrite && !tmp.lock()) {
                 qCWarning(KCONFIG_CORE_LOG) << "couldn't lock global file";
 
                 // unlock the local config if we're returning early
-                if (d->mBackend->isLocked()) {
-                    d->mBackend->unlock();
+                if (d->mBackend.isLocked()) {
+                    d->mBackend.unlock();
                 }
 
                 d->bDirty = true;
                 return false;
             }
-            if (!tmp->writeConfig(utf8Locale, d->entryMap, KConfigIniBackend::WriteGlobal)) {
+            if (!tmp.writeConfig(utf8Locale, d->entryMap, KConfigIniBackend::WriteGlobal)) {
                 d->bDirty = true;
             }
-            if (tmp->isLocked()) {
-                tmp->unlock();
+            if (tmp.isLocked()) {
+                tmp.unlock();
             }
         }
 
         if (writeLocals) {
-            if (!d->mBackend->writeConfig(utf8Locale, d->entryMap, KConfigIniBackend::WriteOptions())) {
+            if (!d->mBackend.writeConfig(utf8Locale, d->entryMap, KConfigIniBackend::WriteOptions())) {
                 d->bDirty = true;
             }
         }
-        if (d->mBackend->isLocked()) {
-            d->mBackend->unlock();
+        if (d->mBackend.isLocked()) {
+            d->mBackend.unlock();
         }
     }
 
@@ -639,9 +662,9 @@ void KConfigPrivate::changeFileName(const QString &name)
 
     bSuppressGlobal = (file.compare(*sGlobalFileName, sPathCaseSensitivity) == 0);
 
-    mBackend->setFilePath(file);
+    mBackend.setFilePath(file);
 
-    configState = mBackend->accessMode();
+    configState = mBackend.accessMode();
 }
 
 void KConfig::reparseConfiguration()
@@ -670,6 +693,13 @@ void KConfig::reparseConfiguration()
         d->parseGlobalFiles();
     }
 
+#ifdef Q_OS_WIN
+    // Parse the windows registry defaults if desired
+    if (d->openFlags & ~KConfig::SimpleConfig) {
+        d->parseWindowsDefaults();
+    }
+#endif
+
     d->parseConfigFiles();
 }
 
@@ -704,17 +734,17 @@ void KConfigPrivate::parseGlobalFiles()
     //    qDebug() << "parsing global files" << globalFiles;
 
     Q_ASSERT(entryMap.empty());
-    const ParseCacheKey key = {globalFiles, locale};
-    auto data = sGlobalParse->localData().object(key);
-    QDateTime newest;
+
+    ParseCacheTimestamp timestamp;
+    timestamp.reserve(globalFiles.count());
     for (const auto &file : globalFiles) {
-        const auto fileDate = QFileInfo(file).lastModified(QTimeZone::UTC);
-        if (fileDate > newest) {
-            newest = fileDate;
-        }
+        timestamp << QFileInfo(file).lastModified(QTimeZone::UTC).toMSecsSinceEpoch();
     }
+
+    const ParseCacheKey key = {globalFiles, locale};
+    auto data = sGlobalParse.localData().object(key);
     if (data) {
-        if (data->parseTime < newest) {
+        if (data->timestamp != timestamp) {
             data = nullptr;
         } else {
             entryMap = data->entries;
@@ -730,15 +760,28 @@ void KConfigPrivate::parseGlobalFiles()
             parseOpts |= KConfigIniBackend::ParseDefaults;
         }
 
-        QExplicitlySharedDataPointer<KConfigIniBackend> backend(new KConfigIniBackend);
-        backend->setFilePath(file);
-        if (backend->parseConfig(utf8Locale, entryMap, parseOpts) == KConfigIniBackend::ParseImmutable) {
+        KConfigIniBackend backend;
+        backend.setFilePath(file);
+        if (backend.parseConfig(utf8Locale, entryMap, parseOpts) == KConfigIniBackend::ParseImmutable) {
             break;
         }
     }
-    sGlobalParse->localData().insert(key, new ParseCacheValue({entryMap, newest}));
+    sGlobalParse.localData().insert(key, new ParseCacheValue({entryMap, timestamp}));
 }
 
+#ifdef Q_OS_WIN
+void KConfigPrivate::parseWindowsDefaults()
+{
+    if (fileName.isEmpty() || QCoreApplication::organizationName().isEmpty()) {
+        return;
+    }
+    auto registryKey =
+        QStringLiteral("SOFTWARE\\%1\\%2")
+            .arg(QCoreApplication::organizationName(), fileName.endsWith(QStringLiteral("rc")) ? fileName.left(fileName.length() - 2) : fileName);
+    WindowsRegistry::parse(registryKey, entryMap);
+}
+#endif
+
 void KConfigPrivate::parseConfigFiles()
 {
     // can only read the file if there is a backend and a file name
@@ -769,7 +812,7 @@ void KConfigPrivate::parseConfigFiles()
                 }
             }
         } else {
-            files << mBackend->filePath();
+            files << mBackend.filePath();
         }
         if (!isSimple()) {
             files = QList<QString>(extraFiles.cbegin(), extraFiles.cend()) + files;
@@ -779,8 +822,8 @@ void KConfigPrivate::parseConfigFiles()
 
         const QByteArray utf8Locale = locale.toUtf8();
         for (const QString &file : std::as_const(files)) {
-            if (file.compare(mBackend->filePath(), sPathCaseSensitivity) == 0) {
-                switch (mBackend->parseConfig(utf8Locale, entryMap, KConfigIniBackend::ParseExpansions)) {
+            if (file.compare(mBackend.filePath(), sPathCaseSensitivity) == 0) {
+                switch (mBackend.parseConfig(utf8Locale, entryMap, KConfigIniBackend::ParseExpansions)) {
                 case KConfigIniBackend::ParseOk:
                     break;
                 case KConfigIniBackend::ParseImmutable:
@@ -791,10 +834,10 @@ void KConfigPrivate::parseConfigFiles()
                     break;
                 }
             } else {
-                QExplicitlySharedDataPointer<KConfigIniBackend> backend(new KConfigIniBackend);
-                backend->setFilePath(file);
+                KConfigIniBackend backend;
+                backend.setFilePath(file);
                 constexpr auto parseOpts = KConfigIniBackend::ParseDefaults | KConfigIniBackend::ParseExpansions;
-                bFileImmutable = backend->parseConfig(utf8Locale, entryMap, parseOpts) == KConfigIniBackend::ParseImmutable;
+                bFileImmutable = backend.parseConfig(utf8Locale, entryMap, parseOpts) == KConfigIniBackend::ParseImmutable;
             }
 
             if (bFileImmutable) {
@@ -926,11 +969,11 @@ void KConfig::deleteGroupImpl(const QString &aGroup, WriteConfigFlags flags)
 bool KConfig::isConfigWritable(bool warnUser)
 {
     Q_D(KConfig);
-    bool allWritable = d->mBackend->isWritable();
+    bool allWritable = d->mBackend.isWritable();
 
     if (warnUser && !allWritable) {
         QString errorMsg;
-        errorMsg = d->mBackend->nonWritableErrorMessage();
+        errorMsg = d->mBackend.nonWritableErrorMessage();
 
         // Note: We don't ask the user if we should not ask this question again because we can't save the answer.
         errorMsg += QCoreApplication::translate("KConfig", "Please contact your system administrator.");
