@@ -258,7 +258,9 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray &val
     case QMetaType::Int:
     case QMetaType::UInt:
     case QMetaType::LongLong:
-    case QMetaType::ULongLong: {
+    case QMetaType::ULongLong:
+    case QMetaType::Long:
+    case QMetaType::ULong: {
         QVariant tmp = value;
         if (!tmp.convert(aDefault.metaType())) {
             tmp = aDefault;
@@ -377,6 +379,8 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray &val
         }
         return date;
     }
+    case QMetaType::QTime:
+        return QTime::fromString(QString::fromUtf8(value));
     case QMetaType::QColor:
     case QMetaType::QFont:
         qCWarning(KCONFIG_CORE_LOG) << "KConfigGroup::readEntry was passed GUI type '" << aDefault.typeName()
@@ -908,6 +912,8 @@ void KConfigGroup::writeEntry(const char *key, const QVariant &value, WriteConfi
     case QMetaType::Bool:
     case QMetaType::LongLong:
     case QMetaType::ULongLong:
+    case QMetaType::Long:
+    case QMetaType::ULong:
         data = value.toString().toUtf8();
         break;
     case QMetaType::QVariantList:
@@ -980,6 +986,10 @@ void KConfigGroup::writeEntry(const char *key, const QVariant &value, WriteConfi
         writeEntry(key, list, flags);
         return;
     }
+    case QMetaType::QTime: {
+        data = value.toTime().toString().toUtf8();
+        break;
+    }
     case QMetaType::QDateTime: {
         const QDateTime rDateTime = value.toDateTime();
 
