@@ -11,6 +11,8 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
+// clazy:excludeall=use-static-qregularexpression
+
 #include "KConfigSourceGenerator.h"
 
 #include <QRegularExpression>
@@ -261,7 +263,7 @@ void KConfigSourceGenerator::createParentConstructorCall()
             stream() << " config ";
         }
     }
-    if (!parseResult.cfgFileName.isEmpty()) {
+    if (parseResult.cfgStateConfig || !parseResult.cfgFileName.isEmpty()) {
         stream() << ") ";
     }
     stream() << ")\n";
